	SUBROUTINE CONVLV2DG(ARRAYIN,RADIUSIN,SIZE,SMEAR,NPTS_S,
	1ARRAYOUT,NPTS,NPTS_C,NANGLES)
C
C       This subroutine convolves the one-dimensional array ARRAYIN with 
C  the one-dimensional array SMEAR.  The values in ARRAYIN and SMEAR are 
C  derived from averages along elliptical contours of objects in 
C  two-dimensional files, and are assumed to be symmetric about the first 
C  element (index = 0).  This subroutine is useful for convolving galaxy 
C  surface brightness distributions with a seeing distribution when there 
C  are few pixels resolving the center of the galaxy.  The information in 
C  ARRAYIN and RADIUSIN can be arranged so that the step size in radius 
C  is smallest at the center of the galaxy.  There are no restrictions on 
C  how the step size is determined but it should be set up in the calling 
C  program.
C  
C				Donald H. Gudehus, March, l984
C
C	ARRAYIN	 (C)	The array containing the information to be
C			convolved
C	RADIUSIN (C)	The array containing the corresponding radii
C			for each element of ARRAYIN
C	SIZE	 (C)	The dimension of the above 2 arrays
C	SMEAR	 (C)	The smearing function taken to be symmetric
C			about the first index.  The index is equal to
C			the radius measured in pixels.
C	NPTS_S	 (C)	The number of data points or pixels in the array
C			SMEAR and the dimension of the array
C	ARRAYOUT (R)	The output array
C	NPTS	 (C)	The dimension of the array ARRAYOUT and the 
C			maximum radius measured in pixels plus 1.
C	NPTS_C	 (C)	The number of points or pixels to be convolved.
C			If the entire range is to be convolved it should
C			equal NPTS but if a saving in time is desired it
C			can be less.  If NPTS_C < NPTS, ARRAYOUT will
C			contain the unconvolved information for indexes
C			> NPTS_C - 1.
C	NANGLES	(C)	The number of different angles at which the
C			function is sampled;  36 is a good number.
C			72 is the current maximum.
C
	INTEGER SIZE
	DIMENSION ARRAYIN(0:SIZE-1),RADIUSIN(0:SIZE-1),SMEAR(0:NPTS_S-1)
	1,ARRAYOUT(0:NPTS-1)
	DIMENSION COSINE(0:71)
	REAL*8 NORM,TEMP
C
C  Check on the limit for NANGLES
	IF (NANGLES.GT.72) NANGLES=72
C  Load ARRAYOUT with the information from ARRAYIN to cover the case of
C  NPTS_C < NPTS.  Since SIZE is greater than NPTS, we must interpolate 
C  values from ARRAYIN to put into ARRAYOUT.
	ARRAYOUT(0)=ARRAYIN(0)
	I_OUT=1
	I_IN=1
	DO WHILE (I_OUT.LE.NPTS-1.AND.I_IN.LE.SIZE-1)
	  IF (INT(RADIUSIN(I_IN)).GE.I_OUT) THEN
	    ARRAYOUT(I_OUT)=
	1   (I_OUT-RADIUSIN(I_IN-1))/
	2   (RADIUSIN(I_IN)-RADIUSIN(I_IN-1))
	3   *(ARRAYIN(I_IN)-ARRAYIN(I_IN-1))
	4   +ARRAYIN(I_IN-1)
	    I_OUT=I_OUT+1
	  END IF
	  I_IN=I_IN+1
	END DO
C  Compute the cosines for the loop with index K
	DO I=0,NANGLES-1
	  COSINE(I)=COS((360.0/FLOATJ(NANGLES))*FLOATJ(I)/57.2957795)
	END DO
C  Begin the loop for positioning the smearing function relative to the
C  data array.  Start with the center of the smearing function
C  (index = 0) at the beginning of the data array (index = 0).
	IHIGH=0  ! Initialization
	ICEN=0
	DO 100 JRADIUS=0,NPTS_C-1
	  IF (JRADIUS.EQ.NPTS-1) GO TO 100 !  Saves time and increases accuracy
					   !  by skipping this loop on the last
					   !  point.
C  Find the range in index I for the loop which adds contributions to the
C  JRADIUSth location of the output array.  The range is decreased as
C  one proceeds higher in JRADIUS because ARRAYIN is finite and because the
C  smearing operation must be kept symmetric about the smearing 
C  function.  Start the search for IHIGH and ILOW at the previously
C  determined value in order to save time.
	  DO WHILE (RADIUSIN(IHIGH).LE.MIN(JRADIUS+NPTS_S-1,NPTS-1))
	    IHIGH=IHIGH+1
	  END DO
	  IHIGH=IHIGH-1
	  DO WHILE (RADIUSIN(ICEN).LE.JRADIUS)
	    ICEN=ICEN+1
	  END DO
	  ICEN=ICEN-1
	  IF (IHIGH.GE.SIZE) THEN
	    IHIGH=SIZE-1  ! Doesn't read beyond the end of ARRAYIN
	  END IF
	  ILOW=ICEN+1
	  IF (ILOW.GE.SIZE) ILOW=SIZE-1
	  IF (ILOW.GT.IHIGH) ILOW=IHIGH
C  Compute the contribution of the central point of the smearing 
C  function for the JRADIUSth location.
	  TEMP=FLOATJ(NANGLES)*ARRAYIN(ICEN)*
	1 SMEAR(0)*0.5*(RADIUSIN(ILOW)-JRADIUS)**2  ! 0.5=PI/2*PI
	  NORM=SMEAR(0)*0.5*(RADIUSIN(ILOW)-JRADIUS)**2
	  DO 50 I=ILOW,IHIGH  !  I references RADIUSIN
	    SRADIUS=RADIUSIN(I)-JRADIUS
	    DELTARADIUS=RADIUSIN(I+1)-RADIUSIN(I)
	    DO 40 K=0,NANGLES-1  !  Increments by 360/NANGLES degrees
				 !  around the center of the smearing
				 !  function
C  Get the distance to the center of the galaxy in pixels
	      RADIUS=SQRT((SRADIUS)**2+FLOATJ(JRADIUS)**2+
	1     2*(SRADIUS)*FLOATJ(JRADIUS)*COSINE(K))
		!  + sign here because this angle
		!  = 180 - the interior angle of the
		!  triangle with sides JRADIUS, SRADIUS, and 
		!  RADIUS
C  Find the index of ARRAYIN and RADIUSIN
	      IRADIUS=0  ! Initialization
	      DO WHILE (RADIUSIN(IRADIUS).LE.RADIUS)
	        IRADIUS=IRADIUS+1
	      END DO
	      IRADIUS=IRADIUS-1
C  Now interpolate to get the correct value from ARRAYIN
	      IF (RADIUS-RADIUSIN(IRADIUS).EQ.0.0) THEN
	        A_VALUE=SRADIUS*ARRAYIN(IRADIUS)
	      ELSE
	        A_VALUE=SRADIUS*(
	1       (RADIUS-RADIUSIN(IRADIUS))/
	2       (RADIUSIN(IRADIUS+1)-RADIUSIN(IRADIUS))
	3       *(ARRAYIN(IRADIUS+1)-ARRAYIN(IRADIUS))
	4       +ARRAYIN(IRADIUS))
	      END IF
C  Interpolate SMEAR
	      SMEARINTRP=(RADIUSIN(I)-INT(RADIUSIN(I)))*
	1     (SMEAR(INT(SRADIUS)+1)-SMEAR(INT(SRADIUS)))
	2     +SMEAR(INT(SRADIUS))
	      TEMP=TEMP+A_VALUE*SMEARINTRP*
	1     DELTARADIUS
  40	    CONTINUE
	    NORM=NORM+(SRADIUS)*SMEARINTRP*DELTARADIUS
  50	  CONTINUE
	  ARRAYOUT(JRADIUS)=TEMP/(FLOATJ(NANGLES)*NORM)
 100	CONTINUE
	RETURN
	END
